/*
    Copyright (c) 2012 Centre for Advanced Internet Architectures,
    Swinburne University of Technology.

    Authors:    Grenville Armitage  (garmitage@swin.edu.au)
                Chris Holman        (6963420@student.swin.edu.au)

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
    1. Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

    THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.

    
    
    Aduino l3dgecomm example usage v0.1
    
*/
#include <SPI.h>
#include <EthernetUdp.h>
#include <l3dgecomm.h>
#include <Time.h>

// This example uses a servo for actions
#include <Servo.h>

byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xF9, 0xFF };
IPAddress ip (10, 0, 0, 2);

IPAddress l3dgeIP (10, 0 , 0, 1);
unsigned int l3dgePort = 27960;

l3dgecomm * L;

Servo serv;

void setup()
{
    Ethernet.begin(mac,ip);
    L = new l3dgecomm(l3dgeIP, l3dgePort);
    serv.attach(5);
}

void loop()

{
  char buf[20] ;
  unsigned int hostId = 1;
  Action * act;
  
  // Set the names for one of the metrics used
  L->sendUpdate(hostId,L3DGE_METRIC_SPIN,L3DGE_FIELD_NAME, "Temperature");
  
  // Send an update to spin entity 1
  sprintf(buf, "%i", analogRead(0)/10 );
  L->sendUpdate(hostId,L3DGE_METRIC_SPIN,L3DGE_FIELD_RATE, buf);
    sprintf(buf, "%i Degrees C", analogRead(0) / 100 + 40 );
  L->sendUpdate(hostId,L3DGE_METRIC_SPIN,L3DGE_FIELD_RATE, buf);
  
  // Send an update to change the bounce height of entity 2
  sprintf(buf, "%i", analogRead(1)/10 );
  L->sendUpdate(2,L3DGE_METRIC_BOUNCE_HEIGHT, L3DGE_FIELD_RATE, buf);
  
  // Send an update to change the colour of host 3
  sprintf(buf, "%i", analogRead(2)/4 );
  //L->sendUpdate(3,L3DGE_METRIC_COLOUR,L3DGE_FIELD_RATE, buf);
  
  // Send a request for the server to reread the position of entity 1
  L->updatePos(1);
  
  // Check for any incoming packets
  // IMPORTANT: This will also check for any invalidtoken packets
  while ( act =  L->checkPackets())
  {
    // If the action is for entity 1, set the servo position to 
    // 10 times the tool ID
    if ( act->entityId == 1)
        serv.write( act->toolId * 10);
  }
  
  // Pause a bit after each loop; the arduino mega is only capable of
  // 100pps or so; 
  delay(500);
}
