/*

arduino-l3dgecomms is distributed under the FreeBSD license.

Copyright (c) 2012 Centre for Advanced Internet Architectures,
Swinburne University of Technology. 

Author: Greville Armitage ( garmitage@swin.edu.au ) 
        Chris Holman ( 6963420@student.swin.edu.au )

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.


arudino-l3dgecomms v0.2


*/

#include <Arduino.h>
#include <Ethernet.h> 
#include <EthernetUdp.h>
#include "l3dgecomm.h"
#include <Time.h>

l3dgecomm::l3dgecomm(IPAddress connIp, unsigned int connPort)
{
    char * tkn;
    char packetBuffer[UDP_TX_PACKET_MAX_SIZE];
    int pktSize;

    serv = 100;
    
    // Clear our tokens first
    strncpy(token, "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0", 17);
    strncpy(outputToken, "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0", 17);
    
    // Initialise ethernet connection and UDP socket
    localPort = 27960;
    ip = connIp;
    port = connPort;
    conn.begin(localPort); 
    
    // Initialise l3dgecomm connection and request token
    conn.beginPacket(ip, port);
    conn.write(msgGetToken);
    conn.endPacket();
    
    // Set time token was sent
    getTokenSent = now();
    
    /*
     * Wait for the token reply to be received, and store it
     * This bit could be removed and left up to the checkPackets() function
     * I haven't moved it so that we'll always have a token before we begin 
     * trying to send l3dgecomm data
     */
    while (true)
    {
    
        // If we've been waiting more than 5 seconds 
        if ( now() - getTokenSent > 5 )
        {
            // Invalidate previous token and send token request
            token[0]  = '\0';
            getTokenSent = now();
            conn.beginPacket(ip, port);
            conn.write(msgGetToken);
            conn.endPacket();
            
        }
        
        // If there's no packet waiting for us, delay a bit, 
        if ( !conn.parsePacket())
            { delay(1); continue; }
            
        // Copy packet contents in to buffer
        conn.read(packetBuffer,UDP_TX_PACKET_MAX_SIZE);
      
        // Check that the packet starts with 0xFF0xFF0xFF0xFF
        if ( strncmp(packetBuffer, fourFs, 4) )
        {
            continue;
        }
        // Check that we didn't get invalid
        if (strstr(packetBuffer, msgInvalid))
        {
            continue;
        }

        // Check if it's a token reply - if yes, we can break out of the loop!
        if (tkn = strstr(packetBuffer, msgReply))
        {
            tkn +=  strlen(msgReply);
        
            strncpy(token, tkn, 16);
            token[16] = '\0';
            break;
        }
    }
    
}

Action * l3dgecomm::checkPackets()
{
    char * tkn;
    char packetBuffer[UDP_TX_PACKET_MAX_SIZE];
    int pktSize;
    IPAddress remote_ip;
    unsigned int remote_port;
    
    // While there are still packets left to check ...
    while (pktSize = conn.parsePacket())
    {
        remote_ip = conn.remoteIP();
        remote_port = conn.remotePort();

        conn.read(packetBuffer,UDP_TX_PACKET_MAX_SIZE);
        
        // Check that the packet source address is correct
        if ( memcmp( &ip, &remote_ip, 4) != 0 )
            continue;
        if ( remote_port != port )
            continue;
        
        // Check if the message is 'invalidtoken'
        if ( strstr(packetBuffer, msgInvalid ) == packetBuffer + 4)
        {
            token[0] = '\0';
            // If we've been waiting more than 5 seconds 
            if ( now() - getTokenSent > 5 )
            {
                // Initialise l3dge connection and request token
                token[0]  = '\0';
                getTokenSent = now();
                conn.beginPacket(ip, port);
                conn.write(msgGetToken);
                conn.endPacket();
                
            }
            continue;
        }
        
        // Check if it's a token reply 
        if (tkn = strstr(packetBuffer, msgReply))
        {
            tkn +=  strlen(msgReply);
        
            strncpy(token, tkn, 16);
            token[16] = '\0';
            continue;
        }
        
        // Check that the token is correct
        
        
        // Check for an output daemon toekn reqeust
        if ( strstr(packetBuffer, msgTokenRequest) == packetBuffer + 4 )
        {
            char replyBuffer[500];
            Serial.println("Got token request");
            // If we dont' have an existing token, dont' do anything
            if ( token[0] == '\0')
                continue;
            
            // XXX: Check for valid token 
            
            strncpy(replyBuffer, fourFs, 4);
            strncpy(replyBuffer + 4, msgOutputToken, 12 );
            strcpy(outputToken, "1234567812345678");
            strncpy(replyBuffer + 4 + 12, token, 16);
			strncpy(replyBuffer + 4 + 12 + 16, " ", 1);
            strncpy(replyBuffer + 4 + 12 + 16 + 1, outputToken, 16);
            replyBuffer[4+12+16+1+16] = '\0';
            //Serial.println(replyBuffer);
            //Serial.println(token);
            //Serial.println(outputToken);
            
            
            conn.beginPacket(ip, port);
            conn.write(replyBuffer);
            conn.endPacket();
        }
        
        // We've received an action
        if ( strstr(packetBuffer, msgAction) == packetBuffer + 4)
        {
            // XXX: Handle this outside of the library; add it to an action queue or something
            char * ptr;
            unsigned int entity;
            unsigned int ackNum;
            Action * retVal;
            char replyBuffer[50];
            retVal = (Action *) malloc(sizeof(Action));
            
            ptr = packetBuffer + 4 + 7 + 1; // FourFs + "action " + ~
            
            // Check for valid outputToken
            if (strncmp(ptr, outputToken, 16) )
                // Invalid token
            { 
				Serial.println("Invalid token received for action"); 
				// Construct and send the acknowledgement
				sprintf(replyBuffer, "%sinvalidtoken", fourFs );
				conn.beginPacket(ip, port);
				conn.write(replyBuffer);
				conn.endPacket();
				continue; 
			}
            
            // Move pointer to end of token
            ptr += 16; 
            
            // Move through the packet payload looking for our values
            // XXX: A malformed packet could crash or exploit this
            //      Should put in a check for length of string
            ptr = strchr(ptr, '~') + 1; // Get ack num
            ackNum = atoi(ptr);
            ptr++;
            ptr = strchr(ptr, '~') + 1; // Make sure we're at the right spot anyway
            retVal->entityId = atoi(ptr);
            ptr++;
            ptr = strchr(ptr, '~') + 1;
            retVal -> toolId = atoi(ptr);
            ptr++;
            ptr = strchr(ptr, '~') + 1;
            strncpy(retVal->entity, ptr, UDP_TX_PACKET_MAX_SIZE);

            
            // Construct and send the acknowledgement
            sprintf(replyBuffer, "%s%s%s %i", fourFs, msgAck, outputToken, ackNum);
            conn.beginPacket(ip, port);
            conn.write(replyBuffer);
            conn.endPacket();
            
            return retVal;
            
        }
        

    }
    
    // No more packets left to process, return null
    return NULL;
    
}

void l3dgecomm::sendUpdate( unsigned int hostId, unsigned int metric, const char * fieldType, const char* value)
{
    char replyBuffer[UDP_TX_PACKET_MAX_SIZE];
    
    // Check that we have a valid token to use
    if ( token[0] == '\0' )
        return;
        
    // Assemble update payload
    sprintf(replyBuffer, "%sl3dge %s ~%03i~%d~%s~%s~",
      fourFs,
      token,
      hostId,
      metric,
      fieldType,
      value
    );
    
    // Send update packet
    conn.beginPacket(ip, port);
    conn.write(replyBuffer);
    conn.endPacket();
}

void l3dgecomm::updatePos( unsigned int hostId)
{
    // Call the multi-update function with a single element array
    unsigned int i[1];
    i[0] = hostId;
    updateMultiplePos(i, 1);
}

void l3dgecomm::updateMultiplePos(unsigned int hostIds[], unsigned int numHosts)
{
    int i;
    char * ids; //[UDP_TX_PACKET_MAX_SIZE - 26];
    char tempInt[5];
    
    // Check that we have a token to use
    if ( token[0] == '\0' )
        return;
    
    // Check that we aren't going to have too many updates 
    if ( numHosts * 5 > UDP_TX_PACKET_MAX_SIZE - 26 ) 
    { Serial.println("Attempted to update too many nodes. Increase UDP_TX_PACKET_MAX_SIZE"); return; }
    
    ids = (char*)malloc(sizeof(char)*5*numHosts);
    
    // String together each of the IDs, separated by "~"
    strcpy(ids, "~");
    for ( i = 0; i < numHosts; i++)
    {
        sprintf(tempInt, "%i", hostIds[i]);
        ids = strcat(ids, tempInt);
        ids = strcat(ids, "~");
    }
    
    // Assemple update payload
    char replyBuffer[UDP_TX_PACKET_MAX_SIZE];
    sprintf(replyBuffer, "%s move %s %s",
      fourFs,
      token,
      ids
    );
    
    // Send packet
    conn.beginPacket(ip, port);
    conn.write(replyBuffer);
    conn.endPacket();    
 
    
    free(ids);

}