/*

arduino-l3dgecomms is distributed under the FreeBSD license.

Copyright (c) 2012 Centre for Advanced Internet Architectures,
Swinburne University of Technology. 

Author: Greville Armitage ( garmitage@swin.edu.au ) 
        Chris Holman ( 6963420@student.swin.edu.au )

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.



  arduino-l3gecomms v0.2

*/


#ifndef l3dgecomm_h
#define l3dgecomm_h

/* 
 * Todo:
 *      Implement handling of invalidtoken
 *      Add sendFeedback method for sending messages to onscreen
 *      Implement checking for if we've already ack'd a packet
 *      Add a wrapper function such that both r and v are set at the same time
 */

#include <Arduino.h>
#include <SPI.h>
#include <Ethernet.h> 
#include <EthernetUdp.h>
#include <Time.h>  

// Increase max transmission size
#define UDP_TX_PACKET_MAX_SIZE 150

// Some useful strings 
const  char msgGetToken[] = "\xFF\xFF\xFF\xFFgettoken";
const  char fourFs[] = "\xFF\xFF\xFF\xFF";
const  char msgInvalid[] = "invalidtoken";
const  char msgReply[] = "tokenreply:";   
const  char msgTokenRequest[] = "gettoken";
const  char msgOutputToken[] = "outputtoken ";
const  char msgAction[] = "action ";
const  char msgAck[] = "ack ";

const char L3DGE_FIELD_RATE[] = "r";
const char L3DGE_FIELD_VALUE[] = "v";
const char L3DGE_FIELD_NAME[] = "n";

const unsigned int L3DGE_METRIC_SPIN = 1;
const unsigned int L3DGE_METRIC_BOUNCE_HEIGHT = 2;
const unsigned int L3DGE_METRIC_SCALE = 3;
const unsigned int L3DGE_METRIC_ROLL = 4;
const unsigned int L3DGE_METRIC_COLOUR = 5;
const unsigned int L3DGE_METRIC_BOUNCE_RATE = 6;
const unsigned int L3DGE_METRIC_SOUND = 7;
const unsigned int L3DGE_METRIC_ALT_SKIN = 8;
const unsigned int L3DGE_METRIC_ALT_MODEL = 9;
const unsigned int L3DGE_METRIC_INFO = 0;

struct Action {
    unsigned int entityId;
    unsigned int toolId;
    char entity [UDP_TX_PACKET_MAX_SIZE];
};

class l3dgecomm
{
    
    public:
        l3dgecomm(IPAddress connIp, unsigned int connPort) ;
        void sendUpdate( unsigned int hostId, unsigned int metric, const char * fieldType, const char* value);
        void updatePos( unsigned int hostId);
        void updateMultiplePos(unsigned int hostIds[], unsigned int numHosts);
        Action * checkPackets();
        int serv;
    private:
        EthernetUDP conn;

        char token[17] ;
       //byte sig[] = ;
        unsigned int localPort;
        IPAddress ip;
        unsigned int port;
        
        char outputToken[17];
        time_t getTokenSent;

};

#endif